@@ -26,9 +26,19 @@ sunspec_types = {
     'int16': 'sunsS16',
     'uint32': 'sunsU32',
     'int32': 'sunsS32',
+    'string': 'PackedString',
+    'bitfield16': 'sunsU16',
+    'bitfield32': 'sunsU32',
 }
 
 
+def node_path_string(node):
+    nodes = [node, *node.ancestors()][:-1]
+    names = [node.name for node in reversed(nodes)]
+
+    return ' > '.join(names)
+
+
 class InvalidAccessLevelError(Exception):
     @classmethod
     def build(cls, value, parameter):
@@ -607,8 +617,16 @@ class SizeType:
 class VoidPointerType:
     name = attr.ib(default='void_p')
     type = attr.ib(default='void*')
+    minimum_code = attr.ib(default='((void *) 0)')
+    maximum_code = attr.ib(default='((void *) UINT32_C(0x240000))')
+
+
+@attr.s(frozen=True)
+class PackedStringType:
+    name = attr.ib(default='PackedString')
+    type = attr.ib(default='PackedString')
     minimum_code = attr.ib(default='(0)')
-    maximum_code = attr.ib(default='(UINT32_C(0x240000))')
+    maximum_code = attr.ib(default='(0)')
 
 
 def fixed_width_name(bits, signed):
@@ -652,6 +670,7 @@ types = {
         BooleanType(),
         SizeType(),
         VoidPointerType(),
+        PackedStringType(),
     )
 }
 
@@ -858,13 +877,13 @@ class TableBaseStructures:
             self,
             internal_type,
             internal_name,
-            parameter_uuid,
+            parameter,
             remainder,
             common_initializers,
             meta_initializer,
             setter,
     ):
-        name = self.common_structure_names.get(parameter_uuid)
+        name = self.common_structure_names.get(parameter.uuid)
 
         if name is None:
             if len(self.h_code) > 0:
@@ -872,7 +891,7 @@ class TableBaseStructures:
             if len(self.c_code) > 0:
                 self.c_code.append('')
 
-            formatted_uuid = str(parameter_uuid).replace('-', '_')
+            formatted_uuid = str(parameter.uuid).replace('-', '_')
             name = (
                 f'InterfaceItem_table_common_{internal_type}_{formatted_uuid}'
             )
@@ -903,28 +922,42 @@ class TableBaseStructures:
                     for i, layer in enumerate(layers)
                 }
 
-                full_base_variable = f'&{variable_base}.{remainder}'
+                full_base_variable_name = f'{variable_base}.{remainder}'
+                full_base_variable = f'&{full_base_variable_name}'
 
-            self.common_structure_names[parameter_uuid] = name
+            if internal_type == 'PackedString':
+                meta_entry = []
+                variable_base_length_entry = [
+                    f'.variable_base_length = sizeof({full_base_variable_name}),',
+                ]
+            else:
+                meta_entry = [
+                    f'.meta_values = {{',
+                    meta_initializer,
+                    f'}},',
+                ]
+                variable_base_length_entry = []
+
+            self.common_structure_names[parameter.uuid] = name
             self.h_code.append(
                 f'extern InterfaceItem_table_common_{internal_name} {name};',
             )
             self.c_code.extend([
                 f'#pragma DATA_SECTION({name}, "Interface")',
-                f'// {parameter_uuid}',
+                f'// {node_path_string(parameter)}',
+                f'// {parameter.uuid}',
                 f'InterfaceItem_table_common_{internal_type} const {name} = {{',
                 [
                     f'.common = {{',
                     common_initializers,
                     f'}},',
                     f'.variable_base = {full_base_variable},',
+                    *variable_base_length_entry,
                     f'.setter = {"NULL" if setter is None else setter},',
                     f'.zone_size = {sizes.get("curve_type", 0)},',
                     f'.curve_size = {sizes.get("curve_index", 0)},',
                     f'.point_size = {sizes.get("point_index", 0)},',
-                    f'.meta_values = {{',
-                    meta_initializer,
-                    f'}},',
+                    *meta_entry,
                 ],
                 f'}};',
             ])
@@ -977,9 +1010,10 @@ class TableBaseStructures:
         # TODO: CAMPid 954679654745154274579654265294624765247569765479
         sunspec_getter = 'NULL'
         sunspec_setter = 'NULL'
-        sunspec_variable = 'NULL'
+        sunspec_variable = None
         scale_factor_variable = 'NULL'
         scale_factor_updater = 'NULL'
+        sunspec_model_variable = 'NULL'
 
         if sunspec_point is not None:
             sunspec_type = sunspec_types[
@@ -1005,8 +1039,8 @@ class TableBaseStructures:
                 sunspec_model_variable = f'sunspecInterface.model{model_id}'
                 abbreviation = table_element.abbreviation
                 sunspec_variable = (
-                    f'&{sunspec_model_variable}'
-                    f'.Curve_{curve_index + 1:>02}_{abbreviation}'
+                    f'{sunspec_model_variable}'
+                    f'.Curve_{curve_index:>02}_{abbreviation}'
                 )
 
                 sunspec_getter = '_'.join(
@@ -1077,7 +1111,7 @@ class TableBaseStructures:
         common_structure_name = self.ensure_common_structure(
             internal_type=parameter.internal_type,
             internal_name=types[parameter.internal_type].name,
-            parameter_uuid=parameter.uuid,
+            parameter=parameter,
             remainder=remainder,
             common_initializers=common_initializers,
             meta_initializer=meta_initializer,
@@ -1095,16 +1129,28 @@ class TableBaseStructures:
         if self.include_uuid_in_item:
             maybe_uuid = [f'.uuid = {uuid_initializer(table_element.uuid)},']
 
+        maybe_sunspec_variable_length = []
+        if sunspec_variable is None:
+            sunspec_variable_initializer = 'NULL'
+        else:
+            if parameter.internal_type == 'PackedString':
+                maybe_sunspec_variable_length = [
+                    f'.sunspec_variable_length = sizeof({sunspec_variable}),',
+                ]
+            sunspec_variable_initializer = f'&{sunspec_variable}'
+
         c = [
             f'#pragma DATA_SECTION({item_name}, "Interface")',
+            f'// {node_path_string(table_element)}',
             f'// {table_element.uuid}',
             f'{interface_item_type} const {item_name} = {{',
             [
                 f'.table_common = &{common_structure_name},',
                 f'.can_variable = {can_variable},',
-                f'.sunspec_variable = {sunspec_variable},',
+                f'.sunspec_variable = {sunspec_variable_initializer},',
+                *maybe_sunspec_variable_length,
                 f'.zone = {curve_type if curve_type is not None else "0"},',
-                f'.curve = {curve_index},',
+                f'.curve = {str(int(curve_index - 1))},',
                 f'.point = {0 if point_index is None else point_index},',
                 *maybe_uuid,
             ],
@@ -1425,6 +1471,7 @@ def create_item(
 
     item = [
         f'#pragma DATA_SECTION({item_name}, "Interface")',
+        f'// {node_path_string(parameter)}',
         f'// {item_uuid}',
         f'{interface_item_type} const {item_name} = {{',
         [
